(function() {

'use strict';

/**
 * The ionic-contrib-drawer is fly-out panel that unlike sidemenu appears over the top of content
 */
angular.module('ionic.contrib.drawer', ['ionic'])

.controller('drawerCtrl', ['$element', '$attrs', '$ionicGesture', '$ionicScrollDelegate', '$timeout', '$document', function($element, $attr, $ionicGesture, $ionicScrollDelegate, $timeout, $document) {

  var self = this;

  var el = $element[0];
  var dragging = false;
  var startX, lastX, offsetX, newX, startY, lastY, offsetY, newY;
  var isOpen = false;
  var side;

  // How far to drag before triggering
  var thresholdX = 15;
  // How far from edge before triggering
  var edgeX = 40;

  var LEFT = 0;
  var RIGHT = 1;

  var isTargetDrag = false;
  var isContentDrag = false;
  
  var width = $element[0].clientWidth;
  var height = $element[0].clientHeight;
  var clientWidth = window.innerWidth;

  self.init = function() {
    width = $element[0].clientWidth;
  }

  var enableAnimation = function() {
    $element.addClass('animate');
  };
  var disableAnimation = function() {
    $element.removeClass('animate');
  };

  self.opened = false;

  // Check if this is on target or not
  var isTarget = function(el) {
    while(el) {
      if(el === $element[0]) {
        return true;
      }
      el = el.parentNode;
    }
  };

  var startDrag = function(e) {
    disableAnimation();

    dragging = true;
    offsetX = lastX - startX;
  };

  var startTargetDrag = function(e) {
    disableAnimation();
    
    dragging = true;
    isTargetDrag = true;
    offsetX = lastX - startX;
  };

  var doEndDrag = function(e) {
    startX = null;
    lastX = null;
    offsetX = null;
    isTargetDrag = false;

    if(!dragging) {
      return;
    }

    dragging = false;

    enableAnimation();

    var condition = (newX < (- width / 100 * 20))

    if (side == RIGHT) {
      condition = (newX > (width / 100 * 20))
    }

    ionic.requestAnimationFrame(function() {
      if(condition) {
        self.close()
      } else {
        self.open()
      }
    });
  };

  var doDrag = function(e) {
    // if(e.defaultPrevented) {
    //   return;
    // }

    // if(!lastX) {
    //   startX = e.gesture.touches[0].pageX;
    // }
    // if(!lastY) {
    //   startY = e.gesture.touches[0].pageY;
    // }

    // lastX = e.gesture.touches[0].pageX;
    // lastY = e.gesture.touches[0].pageY;

    // if(!dragging) {

    //   // Dragged 15 pixels and finger is by edge
    //   if(Math.abs(lastX - startX) > thresholdX) {
    //     if(isTarget(e.target)) {
    //       startTargetDrag(e);
    //     } else if(startX < edgeX) {
    //       startDrag(e);
    //     } 
    //   }
    //   // Closing from outside of drawer
    //   else if (isOpen && startX > width){
    //     disableAnimation();
    //     dragging = true;
    //     isContentDrag = true;
    //   }
    // } else {
    //   newX = Math.min(0, (-width + (lastX - offsetX)));
    //   newY = Math.min(0, (-height + (lastY - offsetY)));
    //   var absX = Math.abs(lastX - startX);
    //   var absY = Math.abs(lastY - startY);
    //   if (isContentDrag && lastX < startX) {
    //     var drawerOffsetX = lastX - drawerWidth;
    //     el.style.transform = el.style.webkitTransform = 'translate3d(' + -absX + 'px, 0, 0)';
    //   } 
    //   else if (isTargetDrag && absX > absY + 5){
    //     el.style.transform = el.style.webkitTransform = 'translate3d(' + newX + 'px, 0, 0)';
    //   } else {
    //     el.style.transform = el.style.webkitTransform = 'translate3d(' + newX + 'px, 0, 0)';
    //   }
    // }

    if(dragging) {
      e.gesture.srcEvent.preventDefault();
    }
  };

  side = $attr.side == 'left' ? LEFT : RIGHT;

  if (side == RIGHT) {
    edgeX = clientWidth - edgeX
  }
  $ionicGesture.on('drag', function(e) {
    doDrag(e);
  }, $document);
  $ionicGesture.on('dragend', function(e) {
    $ionicScrollDelegate.freezeAllScrolls(false);
    doEndDrag(e);
  }, $document);


  this.close = function() {
    self.opened = false;

    enableAnimation();
    ionic.requestAnimationFrame(function() {
      if(side === LEFT) {
        el.style.transform = el.style.webkitTransform = 'translate3d(-100%, 0, 0)';
      } else {
        el.style.transform = el.style.webkitTransform = 'translate3d(100%, 0, 0)';
      }
    });

    $document[0].body.classList.remove('drawer-open');
  };

  this.open = function() {
    self.opened = true;

    $document[0].body.classList.add('drawer-open');
    
    enableAnimation();
    ionic.requestAnimationFrame(function() {
      if(side === LEFT) {
        el.style.transform = el.style.webkitTransform = 'translate3d(0%, 0, 0)';
      } else {
        el.style.transform = el.style.webkitTransform = 'translate3d(0%, 0, 0)';
      }
    });
    
  };
}])

.directive('drawer', ['$rootScope', '$ionicGesture', function($rootScope, $ionicGesture) {
  return {
    restrict: 'E',
    controller: 'drawerCtrl',
    link: function($scope, $element, $attr, ctrl) {
      $element.addClass($attr.side);

      ctrl.init();

      $scope.toggleDrawer = function() {
        if (ctrl.opened) {
          ctrl.close();
        } else {
          ctrl.open();
        }
      };

      $scope.openDrawer = function() {
        ctrl.open();
      };
      $scope.closeDrawer = function() {
        ctrl.close();
      };
    }
  }
}])

.directive('drawerClose', ['$rootScope', function($rootScope) {
  return {
    restrict: 'A',
    link: function($scope, $element) {
      $element.bind('click', function() {
        var drawerCtrl = $element.inheritedData('$drawerController');
        drawerCtrl.close();
      });
    }
  }
}]);

})();